@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2016 Hendrik Tews
+ * Copyright (C) 2011 - 2024 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -18,8 +18,6 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
- * 
- * $Id: configuration.ml,v 1.46 2016/01/23 12:57:13 tews Exp $
  *)
 
 
@@ -33,6 +31,17 @@ module U = Unix
 (**/**)
 
 
+(*****************************************************************************
+ *****************************************************************************)
+(** {2 Exceptions} *)
+
+(** Raised when the configuration file has an unexpected version tag. *)
+exception Config_file_wrong_version
+
+(** Raised when the configuration file cannot be read *)
+exception Config_file_invalid
+
+
 (*****************************************************************************
  *****************************************************************************)
 (** {2 Configuration record and global variables} *)
@@ -484,8 +493,8 @@ let read_config_file file_name : t =
     c
   end
   else if string_starts header config_file_header_start
-  then raise(Failure "Incompatible configuration file version")
-  else raise(Failure "Invalid configuration file")
+  then raise Config_file_wrong_version
+  else raise Config_file_invalid
 
 (** Try to load the configuration file at {!config_file_location},
     ignoring all errors. If a valid configuration file is found, the
@@ -501,7 +510,7 @@ let try_load_config_file () =
       (* print_endline "after read"; *)
       res
     with
-      | Failure "Incompatible configuration file version" ->
+      | Config_file_wrong_version ->
 	print_endline "version error";
 	run_message_dialog
 	  ("File " ^ config_file_location ^ 
@@ -510,8 +519,9 @@ let try_load_config_file () =
 	  `WARNING;
 	None
       | _ -> 
-	print_endline "other error";
-	None
+         Printf.printf "Configuration file %s cannot be read.\n"
+           config_file_location;
+	 None
   in
   match copt with
     | None -> ()
@@ -926,12 +936,12 @@ object (self)
 	run_message_dialog
 	  ("No configuration file at " ^ config_file_location ^ "!")
 	  `WARNING
-      | Failure "Incompatible configuration file version" ->
+      | Config_file_wrong_version ->
 	run_message_dialog
 	  ("File " ^ config_file_location ^ 
 	      " is not compatible with this version of Prooftree!")
 	  `WARNING
-      | Failure "Invalid configuration file" ->
+      | Config_file_invalid ->
 	run_message_dialog
 	  ("File " ^ config_file_location ^ " is not a valid Prooftree \
             configuration file!")
