@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2016 Hendrik Tews
+ * Copyright (C) 2011 - 2024 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -18,11 +18,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
- * 
- * $Id: draw_tree.ml,v 1.54 2016/01/23 12:57:13 tews Exp $
  *)
 
-
 (** Layout and drawing of the elements of the proof tree.
 
     Internally a proof tree is organized as an n-ary tree, where the
@@ -35,7 +32,7 @@
     module is completely ignorant. For instance, the root node is
     always a proof goal; proof goal nodes have zero or more successor
     nodes, all of which are proof commands; and, finally, every proof
-    command has precisely one proof-goal successor. These properties
+    command has at most one proof-goal successor. These properties
     are neither assumed nor checked, they hopefully hold, because the
     tree is created in the right way.
 
@@ -71,6 +68,23 @@ open Gtk_ext
 
 (** {2 Utility types and functions} *)
 
+(*****************************************************************************)
+(*****************************************************************************)
+(** {3 turnstile interface} *)
+(*****************************************************************************)
+(*****************************************************************************)
+
+(** Abstract interface for class {!turnstile} for the existential variable
+    record. This class type is used to break the mutual dependency between
+    existential variable records and and the {!turnstile} class.
+ *)
+class type turnstile_interface =
+  object
+    (** Sequent ID, debugging name for sequent elements.
+    *)
+    method id : string
+  end
+
 (*****************************************************************************)
 (*****************************************************************************)
 (** {3 Existential variables} *)
@@ -112,37 +126,67 @@ type existential_status =
     lazily updated in {!Proof_tree.update_existential_status}.
     Therefore, a fully instantiated existential might have status
     {!existential_status.Partially_instantiated} for some time.
+    The external name should only be present for non-instantiated
+    evars. The code however does neither enforce nor rely on this
+    invariant.
+
+   The field [evar_deps] builds the dependency or instantiation tree
+   of existential variables. This tree must be downward closed on
+   registered sequents. That is, any [(state, sequent)] pair present
+   in [evar_sequents] must also be present in all existentials in
+   [evar_deps]. This property is maintained in
+   {!Proof_tree.instantiate_existential} (via
+   {!Proof_tree.propagate_registered_sequents}) and in
+   {!Proof_tree.register_and_update_sequent}.
 *)
 type existential_variable = {
-  existential_name : string;		(** The name *)
-  mutable status : existential_status;	(** instantiation status *)
-  mutable existential_mark : bool;	(** [true] if this existential should 
+  evar_internal_name : string;		(** The internal Coq ID *)
+  mutable evar_external_name : string option;
+					(** The external name if
+                                            not instantiated. *)
+  mutable evar_status : existential_status;
+					(** instantiation status *)
+  mutable evar_mark : bool;		(** [true] if this existential should
 					    be marked in the proof-tree
 					    display *)
-  mutable dependencies : existential_variable list;
+  mutable evar_inst_state : int;
+  				(** State in which the evar got instantiated
+                                    or -1, also -1 for clones. Needed when a
+                                    sequent is registered for an instantiated
+                                    evar. *)
+  mutable evar_deps : existential_variable list;
                                         (** The list of evars that are used 
 					    in the instantiation, 
 					    if instantiated *)
+  mutable evar_sequents : turnstile_interface list Int_map.t
+			(** Sequents that contain this evar and need to be
+                            updated when this evar is instantiated. Empty for
+                            clones and retired proof trees. The map key (int)
+                            is an undo state, it is mapped to all turnstiles
+                            that started to contain this evar in this state.
+                            Thus for undo, one can chop off some upper
+                            portion of this map. To get all turnstiles to
+                            be updated, one has to iterate over all keys. *)
 }
 
 
 (** Filter the non-instantiated existentials from the argument. 
 *)
 let filter_uninstantiated exl =
-  list_filter_rev (fun ex -> ex.status = Uninstantiated) exl
+  list_filter_rev (fun ex -> ex.evar_status = Uninstantiated) exl
 
 (** Filter the partially instantiated existentials from the argument *)
 let filter_partially_instantiated exl =
-  list_filter_rev (fun ex -> ex.status = Partially_instantiated) exl
+  list_filter_rev (fun ex -> ex.evar_status = Partially_instantiated) exl
 
 
 (** Derive the existential status for drawing a node or a connection
     line in the proof tree. 
 *)
 let combine_existential_status_for_tree exl =
-  if List.for_all (fun ex -> ex.status = Fully_instantiated) exl
+  if List.for_all (fun ex -> ex.evar_status = Fully_instantiated) exl
   then Fully_instantiated
-  else if List.exists (fun ex -> ex.status = Uninstantiated) exl
+  else if List.exists (fun ex -> ex.evar_status = Uninstantiated) exl
   then Uninstantiated
   else Partially_instantiated
 
@@ -151,7 +195,7 @@ let combine_existential_status_for_tree exl =
     display purposes.
 *)
 let string_of_existential_list exl =
-  String.concat " " (List.map (fun ex -> ex.existential_name) exl)
+  String.concat " " (List.map (fun ex -> ex.evar_internal_name) exl)
 
 
 (*****************************************************************************)
@@ -443,7 +487,7 @@ end
     It's value is arbitrary for cloned proof trees.
 *)
 class virtual proof_tree_element drawable
-    undo_state debug_name inst_existentials fresh_existentials = 
+    undo_state debug_name new_evars inst_evars =
 object (self)
   inherit [proof_tree_element] doubly_linked_tree
 
@@ -462,12 +506,12 @@ object (self)
   (** The existentials created for this element. Only non-nil when
       this is a proof command.
   *)
-  method fresh_existentials = fresh_existentials
+  method fresh_existentials = new_evars
 
   (** The existentials instantiated by this element. Only non-nil when
       this is a proof command. 
   *)
-  method inst_existentials : existential_variable list = inst_existentials
+  method inst_existentials : existential_variable list = inst_evars
 
   (** Return the state for this sequent. *)
   method undo_state = (undo_state : int)
@@ -525,7 +569,7 @@ object (self)
   val mutable external_windows : external_node_window list = []
 
   (** The set of all existentials for this node. *)
-  val mutable existential_variables = fresh_existentials
+  val mutable existential_variables = new_evars
 
   (** Upward pointer to the layer containing this proof tree. Must be
       set for root nodes.
@@ -538,9 +582,22 @@ object (self)
       the sequent text without existential information, except for the
       head, with contains the current sequent {b with} existential
       info. For uniform treatment of external node windows, the
-      reference is also used for proof commands. There, it holds just
+      field is also used for proof commands. There, it just holds
       one element, the proof command with existentials info.
 
+      This field holds the empty list for additionally spawned subgoals
+      for which the first update sequent command with their initial sequent
+      text has not arrived yet. These sequents are called incomplete.
+      Complete sequents can also receive update sequent commands, but these
+      only update the sequent text with respect to instantiated existential
+      variables. The distinction between complete/incomplete sequents is
+      necessary, because for an update sequent command for a complete sequent,
+      some actions on existential variables are not necessary any more. See
+      {Proof_tree.update_sequent_element}.
+
+      XXX reconsider including existential info everywhere, because 
+      Coq delivers the information now, I believe.
+
       The existential info is omitted from old versions of the sequent
       text, because this info is incorrect for sequents that get
       updated. The problem is that the exisitentials change already
@@ -592,11 +649,11 @@ object (self)
   method existential_variables = existential_variables
 
   (** [inherit_existentials exl] sets this nodes {!attribute:
-      existential_variables} as union of {!fresh_existentials} and
+      existential_variables} as union of {!new_evars} and
       [exl].
   *)
   method inherit_existentials existentials =
-    existential_variables <- List.rev_append fresh_existentials existentials
+    existential_variables <- List.rev_append new_evars existentials
 
   (** The original text content associated with this element. For
       turnstiles this is the sequent text and for proof commands this is the
@@ -1321,6 +1378,14 @@ object (self)
     List.iter (fun ew -> ew#update_content sequent_text_history)
       external_windows
 
+  (** Returns true, if this sequent is complete, i.e., it either is a
+     primary sequent that was sent to prooftree initially with full
+     sequent text or it is a sequent of an additionally spawned
+     subgoal for which the initial update sequent command with the
+     initial sequent text was received already.
+   *)
+  method is_incomplete = sequent_text_history = []
+
   (** Return the pango layout object of {!layout}. Create one if there
       is none.
   *)
@@ -1456,10 +1521,10 @@ let make_layout context =
     It's value is arbitrary for cloned proof trees.
 *)
 class proof_command (drawable_arg : better_drawable) 
-  undo_state command debug_name inst_existentials fresh_existentials =
+  undo_state command debug_name new_evars inst_evars =
 object (self)
   inherit proof_tree_element drawable_arg undo_state debug_name 
-    inst_existentials fresh_existentials 
+    new_evars inst_evars
     as super
 
   (** The part of the proof command that is displayed inside the tree.
@@ -1492,7 +1557,7 @@ object (self)
   (** Height (in pixels) of the rendered proof command text. *)
   val mutable layout_height = 0
 
-                              (***************** inside proof_command_length *)
+                                      (***************** inside proof_command *)
   (** This is a [Proof_command] element, see {!node_kind}. *)
   method node_kind = Proof_command
 
@@ -1522,7 +1587,7 @@ object (self)
     layout_width <- w;
     layout_height <- h
 
-                              (***************** inside proof_command_length *)
+                                      (***************** inside proof_command *)
   (** Set {!displayed_command}. Called from the initializer and when
       the configuration has been changed.
   *)
@@ -1557,7 +1622,7 @@ object (self)
     layout <- make_layout drawable_arg#pango_context;
     super#configuration_updated
 
-                              (***************** inside proof_command_length *)
+                                      (***************** inside proof_command *)
   (** Override {!proof_tree_element.register_external_window} because
       the displayed proof command must be rerendered when an external
       window is registered.
@@ -1574,7 +1639,7 @@ object (self)
     super#delete_external_window win;
     self#render_proof_command
 
-                              (***************** inside proof_command_length *)
+                                      (***************** inside proof_command *)
   (** Draw just this command node.
 
       @param left x-coordinate of the left side of the bounding box of
@@ -1587,8 +1652,8 @@ object (self)
      * Printf.fprintf (debugc()) "DRAW TURN %s l %d t %d x %d y %d\n%!" 
      *   self#debug_name left top x y;
      *)
-    let crea = List.exists (fun e -> e.existential_mark) fresh_existentials in
-    let inst = List.exists (fun e -> e.existential_mark) inst_existentials in
+    let crea = List.exists (fun e -> e.evar_mark) new_evars in
+    let inst = List.exists (fun e -> e.evar_mark) inst_evars in
     if crea || inst
     then begin
       let w = layout_width + 1 * !current_config.subtree_sep in
@@ -1609,7 +1674,7 @@ object (self)
       drawable#rectangle 
 	~x:(x - w/2) ~y:(y - h/2) ~width:w ~height:h ();
 
-                              (***************** inside proof_command_length *)
+                                      (***************** inside proof_command *)
   (** Compute the line offsets for proof-command nodes, see
       {!proof_tree_element.line_offset}
   *)
@@ -1655,17 +1720,20 @@ end
     variable only one {!existential_variable} record is created.
 *)
 let rec clone_existentials ex_hash ex =
-  try Hashtbl.find ex_hash ex.existential_name
+  try Hashtbl.find ex_hash ex.evar_internal_name
   with
     | Not_found -> 
-      let deps = List.map (clone_existentials ex_hash) ex.dependencies in
-      let nex = { existential_name = ex.existential_name;
-		  status = ex.status;
-		  existential_mark = false;
-		  dependencies = deps;
+      let deps = List.map (clone_existentials ex_hash) ex.evar_deps in
+      let nex = {evar_internal_name = ex.evar_internal_name;
+                 evar_external_name = ex.evar_external_name;
+	         evar_status = ex.evar_status;
+	         evar_mark = false;
+                 evar_inst_state = -1;
+	         evar_deps = deps;
+                 evar_sequents = Int_map.empty;
 		}
       in
-      Hashtbl.add ex_hash ex.existential_name nex;
+      Hashtbl.add ex_hash ex.evar_internal_name nex;
       nex
 
 (** Recursively clone all nodes in the given subtree, updating the
@@ -1681,8 +1749,8 @@ let rec clone_tree_node new_pc new_seq ex_hash
   let clone = match node#node_kind with
     | Proof_command -> 
       (new_pc node#content 
-	 (List.map (clone_existentials ex_hash) node#inst_existentials)
 	 (List.map (clone_existentials ex_hash) node#fresh_existentials)
+	 (List.map (clone_existentials ex_hash) node#inst_existentials)
 	 : proof_command :> proof_tree_element)
     | Turnstile -> 
       let ts = new_seq node#id (Some node#content)
