@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2016 Hendrik Tews
+ * Copyright (C) 2011 - 2024 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -18,8 +18,6 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
- * 
- * $Id: input.ml,v 1.43 2016/01/23 12:57:14 tews Exp $
  *)
 
 
@@ -29,16 +27,17 @@
  *****************************************************************************)
 (** {2 Communition Protocol with Proof General}
 
-    The communication protocol with Proof General is mostly one-way:
-    Proof General sends display messages to Prooftree and Prooftree
-    never requests information for the proof-tree display from Proof
-    General. Prooftree sends a notification to Proof General when the
-    proof-tree window is closed. It also sends proof commands to Proof
-    General on request of the user.
+   The communication protocol between Proof General and Prooftree is
+   text based and bidirectional. Proof General sends display messages
+   to Prooftree and Prooftree sends user requests and requests for
+   additional display messages to Proof General. Prooftree sends
+   requests for additional display messages for newly created subgoals
+   and for goals that should be updated, because an existential
+   variable was instantiated.
 
     The communication protocol between Proof General and Prooftree is
     split into two parts: The display messages, which are sent from
-    Proof General to Prooftree and the notification messages, which
+    Proof General to Prooftree and the request messages, which
     are sent from Prooftree to Proof General.
 
     {3 Display Messages}
@@ -55,20 +54,20 @@
     {- a second line containing the display command and the length of
     the additional data sections, if the command has data sections.}
     {- the data sections (if any), where the last character of each
-    data sections is a newline.}
+    data section is a newline.}
     }
 
     All data is UTF-8 encoded. 
 
     Some data sections have a prover specific format. Currently, 
-    [prooftree] only supports Coq.
+    Prooftree only supports Coq.
 
     In the following list 
-    of commands, ``%d'' stands for a positive integer and %s for a string
+    of commands, ``%d'' stands for a positive integer and ``%s'' for a string
     which contains no white space. ``\{cheated | not-cheated\}'' denotes
     the alternative of either ``cheated'' or ``not-cheated''. An
     integer following the keyword state is a state number. An integer
-    following some xxx-bytes denotes the number of bytes of the next
+    following some ``xxx-bytes'' denotes the number of bytes of the next
     <data> section, including the final newline of that <data> section.
     A ``[ \ ]'' at the end of a line denotes line continuation without 
     newline.
@@ -84,12 +83,14 @@
     protocol version NN. If proof assistant PA or version NN is not
     supported, Prooftree displays an error message and exits. The name
     PA might contain arbitrary characters but no quotation mark ( '"' ).
+    This version of Prooftree only supports Coq at protocol version 04.
+    {%html: <p> %}
 
     There must always be exectly one configure message, which must be
     the first message. 
     }
     {-  {v current-goals state %d current-sequent %s \
-    {cheated | not-cheated} {new-layer | current-layer} proof-name-bytes %d \
+    {cheated | not-cheated} proof-name-bytes %d \
     command-bytes %d sequent-text-bytes %d additional-id-bytes %d \
     existential-bytes %d\n\
     <data-proof-name>\n\
@@ -98,7 +99,7 @@
     <data-additional-ids>\n\
     <data-existentials>\n v}
 
-    The [current-goals] command tells [prooftree] about a new proof 
+    The [current-goals] command tells Prooftree about a new proof 
     state with a new set of open goals. This corresponds to either of
     the following cases:
     {ol
@@ -111,62 +112,85 @@
     subgoals}
     {- A new set of proof-tree root goal nodes is associated with the
     current proof. This happens for instance, when Coq transformes
-    open existential variables into proof goals with the command 
-    [Grab Existential Variables].}
+    open existential variables into proof goals with [Unshelve].}
     }
-    
-    For case 1 or case 4 [new-layer] must be given. Otherwise,
-    [current-layer] must be specified and [prooftree] decides with the
-    help of its internal state whether case 2 or 3 applies.
-    
-    For the second and the third case, the set of open goals does not
+
+    Prooftree decides with its internal state which case applies. Case
+    1 and 4 are identically treated internally. Case 1 and 4 are taken
+    if the number of open goals is zero.
+    {%html: <p> %}
+
+    For the second and the third case, the set of open goals in the 4th
+    data section (additional-ids, see below) does not
     need to represent the total set of all open subgoals, but it must
     contain all newly spawned subgoals.
+    {%html: <p> %}
     
     The state number in the [current-goals] command is for undo. It 
     is interpreted as the state that has been reached after processing 
     the current command. 
     [current-sequent %s] denotes the ID of the current sequent. The 
-    cheated flag tells [prooftree] whether the new proof state was 
-    obtained by a cheating command such as [admit] or [sorry]. 
-    The data sections are :
+    cheated flag tells Prooftree whether the new proof state was 
+    obtained by a cheating command such as [admit]. 
+    The data sections are:
     {ol
     {- Full name of the proof}
     {- The proof command that yielded this proof state}
     {- Text of the current sequent}
     {- ID's of additionally open sequents (as space separated 
-    list of strings)}
+    list of strings), containing at least all newly spawned subgoals}
     {- Prover specific information about existential variables.}
     }
     
-    The second data section is ignored for initial proof states.
+    The second data section is ignored for initial proof states (case 1)
+    and new root goal nodes (case 4).
+    {%html: <p> %}
 
-    The text of newly created additional goals other then the 
-    current goal is expected to arrive separately with an 
-    [update-sequent] command.
+    For newly spawned subgoals of this command, Prooftree knows only
+    their sequent ID, but cannot display any sequent text. Such sequents
+    are called incomplete. They stay incomplete, until their first sequent
+    text arrives via an update-sequent command.
+    For each newly spawned subgoal, Prooftree sends a show-goal request
+    to Proof General. Additionally, also for sequents that contain an
+    existential variable that was instantiated by this proof command
+    command, Prooftree sends a show-goal command to Proof General in
+    order to update the display of the affected sequent. Because Proof
+    General and Prooftree progress asynchronously, it may happen, that
+    Proof General processes all these show-goal requests only
+    substantially later. Each show-goal request will be answered by an
+    update-sequent display command (see below) by Proof General.
     }
     {- {v update-sequent state %d sequent %s proof-name-bytes %d \
-    sequent-text-bytes %d\n\
+    sequent-text-bytes %d existential-bytes %d\n\
     <data-proof-name>\n\
-    <data-sequent>\n v}
+    <data-sequent>\n
+    <data-existentials>\n v}
     
     The update sequent command updates the text of some 
     known sequent. Such updates are necessary for newly spawned 
-    subgoals. But also when existential variables get instantiated.
-    
-    The state number is for undo and the sequent ID denotes the 
+    subgoals and for sequents that contain an existantial variable that was
+    instantiated. Update sequent commands are always a response to
+    a show goal request. Update sequent commands may arrive long after
+    the first display command with the same state was processed.
+    {%html: <p> %}
+
+    The state number is the (potentially old) state for which the sequent
+    text was requested by Prooftree. The sequent ID denotes the 
     sequent to update. The data sections are:
+
     {ol
     {- Full name of the proof}
     {- new sequent text}
+    {- Prover specific information about existential variables.}
     }
     }
     {- {v switch-goal state %d sequent %s proof-name-bytes %d\n
     <data-proof-name>\n v}
     
-    Switch goal tells [proftree] that the current goal has changed 
+    Switch goal tells Prooftree that the current goal has changed 
     without changing or solving the old current goal.
-    
+    {%html: <p> %}
+
     The state number is for undo and the only data section is:
     {ol
     {- Full name of the proof}
@@ -178,10 +202,10 @@
     <data-command>\n\
     <data-existentials>\n v}
     
-    [branch-finished] tells [prooftree] the last proof command that 
+    [branch-finished] tells Prooftree the last proof command that 
     closed the current branch. If there are still open subgoals, the 
     proof will hopefully continue with one of them, which is not yet 
-    known. The cheated flag tells [prooftree]
+    known. The cheated flag tells Prooftree
     whether the new proof state was obtained by a cheating command 
     such as [admit] or [sorry]. The data sections are :
     {ol
@@ -194,7 +218,25 @@
     <data-proof-name>\n v}
 
     [proof-complete] tells Prooftree that the current proof has been
-    completed and will further not be updated. The only data section is:
+    completed and will further not be updated. After sending
+    [proof-complete] Proof General blocks processing the queue region 
+    until it receives a [confirm-proof-complete] request message. This
+    way, Proof General will process all [show-goal] request message
+    necessary for this proof before disabling the dependent evar line
+    in Coq and proceeding with the queue region. The [show-goal] messages
+    for incomplete sequents have been sent out when [proof-complete] is
+    processed here, however the corresponding [update-sequent] might
+    arrive later. Therefore Prooftree might send out [show-goal] messages
+    for evar instantiations after receiving [proof-complete] and after
+    receiving all [update-sequent] messages for all incomplete sequents.
+    {%html: <p> %}
+
+    The last open branch should have been closed via [branch-finished]
+    before [proof-complete] arrives, otherwise the proof will be in a
+    strange state.
+    {%html: <p> %}
+
+    The only data section of [proof-complete] is:
     {ol
     {- Full name of the proof}
     }
@@ -202,14 +244,16 @@
     {- {v undo-to state %d\n v}
     
     The state number here is not for undo, it is the undo-state.
-    Undo tells [prooftree] to change the display to the state before 
+    Undo tells Prooftree to change the display to the state before 
     the first command with a state strictly greater than [undo-state]
     has been processed.
     }
     {- {v quit-proof proof-name-bytes %d\n\
     <data-proof-name>\n v}
     
-    Quit closes the window for the indicated proof. 
+    This command tells Prooftree that the user stoped the proof-tree
+    display in Proof General. Prooftree will close the main window for
+    that proof.
     Cloned windows are not closed.
     The only data section is:
     {ol
@@ -218,19 +262,47 @@
     }
     }
 
-    {3 Notification Messages}
+    {3 Request Messages}
 
-    The notification messages are sent from Prooftree to Proof General
-    as a consequence of certain user interactions. There are 3
-    different notification messages: for stopping the proof-tree
-    display, for undo and for sending proof scripts. All notification
+    The request messages are sent from Prooftree to Proof General to
+    request updates of sequents and certain reactions for user
+    interactions. The protocol relies on the fact that [PIPE_BUF] bytes
+    (512 required by POSIX, 4096 on Linux) are transmitted atomically
+    in a pipe. On the receiver side, in Proof General, there are no
+    precautions against not completely transferred first lines
+    (including both newlines) of request messages. 
+
+    All request
     messages are preceeded with a newline and the string 
-    [emacs exec:], followed by a space, for easy recognition in Proof
+    [emacs exec:], followed by a space, and terminated with an additional
+    newline for easy recognition in Proof
     General.
 
-    The remaining part of the messages have the following format.
-
+    The variable parts of the request messages are as follows.
     {ul
+    {- {v show-goal "%s" at %d for "%s" v}
+
+    The first string is the goal id, the number after [at] is a state and
+    the last string is a name of a proof.
+    {%html: <p> %}
+
+    This command requests Proof General to send an [update-sequent] display
+    message for the specified goal in the specified state. All data, the goal
+    id, the state and the proof name, actually makes a round trip to Proof
+    General and comes back in the [update-sequent] command. Note that the
+    state might be an arbitrary previous state of the current proof,
+    occurring in a preceding [current-goals] command.
+    {%html: <p> %}
+
+    Prooftree sends a [show-goal] command for each newly spawned subgoal
+    from a [current-goals] command and for each goal that contains an
+    existential variable that was reported to have gotten instantiated
+    in a [current-goals] command. For the latter case, it may happen that
+    Prooftree finds out about the existential only because of an
+    [update-sequent] command, such that it possibly sends [show-goal]
+    much later than receiving the [current-goals] command that reported the
+    instantiation.
+    }
     {- {v stop-displaying v}
     
     Prooftree sends this message to Proof General when the user closed
@@ -251,15 +323,18 @@
     integer is the length of [<script data>] without the enclosing
     newlines.
     }
+    {- {v confirm-proof-complete "%s" v}
+
+    This message confirms that processing the named proof is complete and
+    that all [show-goal] request messages for this proof have been sent
+    before. This message is required after receiving a [proof-complete]
+    display message.
+    }
     }
 *)
 
-(** Version number of the communication protocol described and
-    implemented by this module.
-*)
-let protocol_version = 3
-
-
+(*****************************************************************************
+ *****************************************************************************)
 
 (** {2 General remarks}
 
@@ -282,25 +357,33 @@ let protocol_version = 3
     [current_parser]. 
 *)
 
+
 (*****************************************************************************
  *****************************************************************************)
 
-
 open Configuration
 open Util
 open Gtk_ext
+open Evar_types
 
 (**/**)
 module U = Unix
 (**/**)
 
 
-(** {2 Module Documentation} 
-    {3 General parsing utilities and parser state}
+(** {2 Module Documentation} *)
+
+(** Version number of the communication protocol described and
+    implemented by this module.
 *)
+(* update documentation for configure message when changed *)
+let protocol_version = 4
 
 
-(** Exception raised if [prooftree] encounters an unknown or malformed command.
+(** {3 General parsing utilities and parser state} *)
+
+
+(** Exception raised if Prooftree encounters an unknown or malformed command.
     The first argument is a description of the error. If the error was caused 
     by an exception, the second argument carries this exception and the 
     execption backtrace until the point where [Protocol_error] was raised.
@@ -316,7 +399,7 @@ exception Protocol_error of string * (exn * string) option
     variables.
 *)
 let parse_existential_info =
-  ref(fun _ -> ([], []) : string -> (string list * (string * string list) list))
+  ref(fun _ -> ([], []) : string -> (evar_info list * string list))
 
 (** Forward pointer to {!message_start}. Initialized in 
     {!setup_input}. The forward pointer is needed, because various 
@@ -379,7 +462,7 @@ let setup_input_backup_channel () =
 (** Input function for reading from the input channel. To make the
     input backup feature work (see option [-tee]) input must always be
     read with this function. Arguments are the same as for {xref
-    stdlib val Pervasives.input}, [local_input buf start len] reads at
+    stdlib val Stdlib.input}, [local_input buf start len] reads at
     most [len] bytes from [stdin] into buffer [buf], starting at
     position [start]. Any material read is immediately written to
     {!Input.input_backup_oc}. Before calling this function,
@@ -391,7 +474,7 @@ let setup_input_backup_channel () =
     {!Input.parse_input} continues parsing with the function stored in
     [current_parser].
 
-    @raise Sys_blocked_io when no more input is available currently
+    @raise Sys_blocked_io when no more input is available currently.
 *)
 let local_input buf start len =
   let read_len = input stdin buf start len in
@@ -407,7 +490,7 @@ let local_input buf start len =
 
 
 (** [get_string_cont s i len cont ()] 
-    fills buffer [s] and continue parsing with [cont]. This
+    fills buffer [s] and continues parsing with [cont]. This
     is a utility function for {!Input.get_string}. [get_string_cont s i
     len cont ()] reads [len - i] bytes from the input channel and
     stores them in [s] at position [i]. When finished it calles
@@ -448,6 +531,36 @@ let get_string len continuation_fn =
   get_string_cont s 0 len continuation_fn ()
 
 
+(******************************************************************************
+ ******************************************************************************
+ * Build Coq evar parser
+ *)
+
+(** {3 Build Coq existential variable info parser} *)
+
+(** Parse Coq existential variable information. Returns a pair. The
+   first element is a list of {!Evar_types.evar_info} describing open
+   evars with their external name and instantiated ones with their
+   dependencies. The second element is the list of internal evar names
+   occuring in the current goal. Uses an OCamllex and OCamlyacc
+   generated parser internally. All parsing exceptions are cought and
+   converted into warning messages. *)
+let coq_evar_parser (input_string : string)
+    : (evar_info list * string list) =
+  try
+    Coq_evar_parser.coq_evar_info Coq_evar_lexer.evar_token
+      (Lexing.from_string input_string)
+  with
+    | e ->
+       if !current_config.debug_mode then
+         Printf.eprintf
+           "Coq evar parser error on \"%s\"\nParser aborts with exception %s\n%!"
+           input_string (Printexc.to_string e);
+       run_message_dialog
+         "Coq evar parser error.\nExistential info might be wrong."
+         `WARNING;
+       ([], [])
+
 (******************************************************************************
  ******************************************************************************
  * configure for "PA" and protocol version NN
@@ -473,8 +586,8 @@ let configure_prooftree proof_assistant pg_protocol_version =
     raise (Protocol_error ("Received a second configure message", None));
   (match proof_assistant with
     | "Coq" -> 
-      parse_existential_info := Coq.coq_parse_existential_info
-    | "HOL Light" -> ()
+      parse_existential_info := coq_evar_parser
+    (* | "HOL Light" -> () *)
     | _ -> 
       raise (Protocol_error ("Unknown proof assistant " ^ proof_assistant,
 			     None))
@@ -498,7 +611,6 @@ let parse_configure com_buf =
 (******************************************************************************
  ******************************************************************************
  * current-goals state %d current-sequent %s {cheated | not-cheated} \
- * {new-layer | current-layer}
  * proof-name-bytes %d command-bytes %d sequent-text-bytes %d \
  * additional-id-bytes %d existential-bytes %d\n\
  * <data-proof-name>\n\
@@ -520,8 +632,6 @@ let parse_configure com_buf =
            line of the command
     @param cheated_string either "cheated" or "not-cheated" from the 
            first line of the command
-    @param layer_string either "new-layer" of "current-layer" from the
-           first line of the command
     @param proof_name name of the current proof
     @param proof_command text of the last proof command (or garbage if 
     this is the first state of the proof)
@@ -532,7 +642,7 @@ let parse_configure com_buf =
 
 *)
 let parse_current_goals_finish state current_sequent_id cheated_string 
-    layer_string proof_name proof_command current_sequent_text 
+    proof_name proof_command current_sequent_text 
     additional_ids_string existentials_string =
   (* Printf.fprintf (debugc()) "PCGF\n%!"; *)
   let cheated_flag = match cheated_string with
@@ -544,25 +654,16 @@ let parse_current_goals_finish state current_sequent_id cheated_string
 		  "Expected \"cheated\" or \"not-cheated\" as 6th word.",
 	       None))
   in
-  let layer_flag = match layer_string with
-    | "new-layer" -> true
-    | "current-layer" -> false
-    | _ ->
-      raise(Protocol_error
-	      ("Parse error in current-goals command. " ^
-		  "Expected \"new-layer\" or \"current-layer\" as 7th word.",
-	       None))
-  in
   let proof_name = chop_final_newlines proof_name in
   let proof_command = chop_final_newlines proof_command in
   let current_sequent_text = chop_final_newlines current_sequent_text in
   let additional_ids_string = chop_final_newlines additional_ids_string in
   let additional_ids = string_split ' ' additional_ids_string in
-  let existentials_string = chop_final_newlines existentials_string in
-  let (ex_uninst, ex_inst) = !parse_existential_info existentials_string in
+  let (evar_info, current_evar_names) =
+    !parse_existential_info existentials_string in
   Proof_tree.process_current_goals state proof_name proof_command cheated_flag
-    layer_flag current_sequent_id current_sequent_text additional_ids 
-    ex_uninst ex_inst;
+    current_sequent_id current_sequent_text additional_ids 
+    evar_info current_evar_names;
   current_parser := !message_start_parser
 
     
@@ -575,10 +676,10 @@ let parse_current_goals_finish state current_sequent_id cheated_string
 let parse_current_goals com_buf =
   check_if_configured ();
   Scanf.bscanf com_buf 
-    (" state %d current-sequent %s %s %s proof-name-bytes %d "
+    (" state %d current-sequent %s %s proof-name-bytes %d "
      ^^ "command-bytes %d sequent-text-bytes %d "
      ^^ "additional-id-bytes %d existential-bytes %d")
-    (fun state current_sequent_id cheated_string layer_string 
+    (fun state current_sequent_id cheated_string
       proof_name_bytes command_bytes sequent_text_bytes additional_id_bytes
       existential_bytes ->
         (*
@@ -601,7 +702,7 @@ let parse_current_goals com_buf =
 			get_string existential_bytes
 			  (fun existentials_string ->
 			    parse_current_goals_finish state current_sequent_id
-			      cheated_string layer_string
+			      cheated_string
 			      proof_name proof_command current_sequent_text
 			      additional_ids_string existentials_string))))))
 
@@ -609,9 +710,10 @@ let parse_current_goals com_buf =
 
 (******************************************************************************
  * update-sequent state %d sequent %s proof-name-bytes %d \
- * sequent-text-bytes %d\n\
+ * sequent-text-bytes %d existential-bytes %d\n
  * <data-proof-name>\n
  * <data-sequent>\n
+ * <data-existentials>\n
  *)
 
 (** {3 Update-sequent command parser} *)
@@ -625,11 +727,17 @@ let parse_current_goals com_buf =
     @param sequent_id ID of sequent to update
     @param proof_name full proof name (as raw data section string)
     @param sequent_text new sequent text (as raw data section string)
+    @param existentials_string prover specific information about 
+    existentials
 *)
-let parse_update_sequent_finish state sequent_id proof_name sequent_text =
+let parse_update_sequent_finish state sequent_id proof_name sequent_text
+      existentials_string =
   let proof_name = chop_final_newlines proof_name in
   let sequent_text = chop_final_newlines sequent_text in
-  Proof_tree.update_sequent state proof_name sequent_id sequent_text;
+  let (evar_info, current_evar_names) =
+    !parse_existential_info existentials_string in
+  Proof_tree.update_sequent state proof_name sequent_id sequent_text
+    evar_info current_evar_names;
   current_parser := !message_start_parser
 
 
@@ -642,14 +750,18 @@ let parse_update_sequent_finish state sequent_id proof_name sequent_text =
 let parse_update_sequent com_buf =
   check_if_configured ();
   Scanf.bscanf com_buf
-    " state %d sequent %s proof-name-bytes %d sequent-text-bytes %d"
-    (fun state sequent_id proof_name_bytes sequent_text_bytes ->
+    (" state %d sequent %s proof-name-bytes %d sequent-text-bytes %d "
+     ^^ "existential-bytes %d")
+    (fun state sequent_id proof_name_bytes sequent_text_bytes
+         existential_bytes ->
       get_string proof_name_bytes
 	(fun proof_name ->
 	  get_string sequent_text_bytes
 	    (fun sequent_text ->
-	      parse_update_sequent_finish state sequent_id 
-		proof_name sequent_text)))
+              get_string existential_bytes
+                (fun existentials_string ->
+	             parse_update_sequent_finish state sequent_id
+		     proof_name sequent_text existentials_string))))
 
 
 
@@ -723,10 +835,10 @@ let parse_branch_finished_finish
   in
   let proof_name = chop_final_newlines proof_name in
   let proof_command = chop_final_newlines proof_command in
-  let existentials_string = chop_final_newlines existentials_string in
-  let (ex_uninst, ex_inst) = !parse_existential_info existentials_string in
+  let (evar_info, current_evar_names) =
+    !parse_existential_info existentials_string in
   Proof_tree.process_branch_finished 
-    state proof_name proof_command cheated_flag ex_uninst ex_inst;
+    state proof_name proof_command cheated_flag evar_info current_evar_names;
   current_parser := !message_start_parser
 
 
@@ -876,7 +988,7 @@ let read_second_line first_line =
     to {!read_second_line} to process the complete display command.
     This function is the entry point into the display-command parser.
     All command parsing functions set {!Input.current_parser} to
-    [read_command_line] when they are finished with their work. This
+    [message_start] when they are finished with their work. This
     way, this function is called again to parse the next command by
     the main parsing loop in {!Input.parse_input}.
 
@@ -888,15 +1000,7 @@ let message_start () =
    * where the number gives the bytes in the next line
    *)
   (* Printf.fprintf (debugc()) "message start\n%!"; *)
-  try
-    get_string 16 read_second_line
-  with
-    | Scanf.Scan_failure _
-    | Failure _ 
-    | End_of_file as e 
-      ->
-      let bt = Printexc.get_backtrace() in
-      raise (Protocol_error ("Parse error", Some(e, bt)))
+  get_string 16 read_second_line
 
 
 (** {3 Main parsing loop, GTK callback and initialization} *)
