@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2016 Hendrik Tews
+ * Copyright (C) 2011 - 2024 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -18,13 +18,12 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
- * 
- * $Id: input.mli,v 1.12 2016/01/23 12:57:14 tews Exp $
  *)
 
 
-(** Reading commands from nonblocking stdin *)
+(* Reading commands from nonblocking stdin *)
 
+open Evar_types
 
 (* for documentation, see input.ml *)
 
@@ -33,6 +32,10 @@
 val configuration_updated : unit -> unit
 
 
+(** Parse Coq existential variable information. *)
+val coq_evar_parser : string -> (evar_info list * string list)
+
+
 (** Initialize this module and setup the GTK main loop callback for
     [stdin]. Puts [stdin] into non-blocking mode.
 *)
