@@ -1,7 +1,7 @@
 (* 
  * prooftree --- proof tree display for Proof General
  * 
- * Copyright (C) 2011 - 2016 Hendrik Tews
+ * Copyright (C) 2011 - 2024 Hendrik Tews
  * 
  * This file is part of "prooftree".
  * 
@@ -18,8 +18,6 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
- * 
- * $Id: proof_window.ml,v 1.67 2016/01/23 12:57:15 tews Exp $
  *)
 
 
@@ -1316,11 +1314,10 @@ object (self)
     new turnstile drawable undo_state sequent_id sequent_text_option
 
   (** Create a new proof command node. *)
-  method new_proof_command undo_state command 
-                           inst_existentials new_existentials =
+  method new_proof_command undo_state command new_evars inst_evars =
     new proof_command drawable undo_state command command 
-      (inst_existentials : existential_variable list)
-      (new_existentials : existential_variable list)
+      (new_evars : existential_variable list)
+      (inst_evars : existential_variable list)
 end
 
 
@@ -1424,7 +1421,7 @@ let rec make_proof_window name geometry_string =
      `I("Show selected", proof_window#show_selected_node);
      `I("Existentials", proof_window#show_existential_window);
      `I("Configuration", show_config_window);
-     `I("Help", show_help_window);
+     `I("Help", show_help_window "");
      `I("About", show_about_window);
      `I("Exit", (fun _ -> exit 0));
     ] in
